﻿define([
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/keys",
    "dojo/dom-class",
    "dojo/on",
    "dojo/when",
    "dijit/Tree",
    "dijit/focus",
    "dijit/_HasDropDown",

    "epi/shell/applicationSettings",

    // epi-cms
    "epi-cms/widget/PersonalizationSelector",
    "epi-cms/dgrid/formatters",
    "./_ContentAreaTreeNodeMixin",

    // resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.blockcontextmenu",
    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.editors.contentarea",
    "dojo/text!./templates/_BlockTreeNode.html"
], function (
    // dojo
    declare,
    lang,
    keys,
    domClass,
    on,
    when,
    Tree,
    focusUtil,
    _HasDropDown,

    applicationSettings,

    // epi-cms
    PersonalizationSelector,
    formatters,
    _ContentAreaTreeNodeMixin,

    // resources
    tooltip,
    resources,
    template
) {

    return declare([Tree._TreeNode, _ContentAreaTreeNodeMixin, _HasDropDown], {
        // tags:
        //      internal

        templateString: template,

        _contextMenuClass: "epi-iconContextMenu",
        isExpandable: false,
        dropDownPosition: ["before"],
        resources: lang.mixin(tooltip, resources.personalize, { shared: resources.shared }),

        buildRendering: function () {
            this.inherited(arguments);

            this.dropDown = new PersonalizationSelector({ model: this.item });
            this.own(this.dropDown);
            this.aroundNode = this.domNode;
            this.set("readOnly", this.item.readOnly);

            domClass.add(this.domNode, "epi-tree-mngr--block");

            var node = this.rolesNode;

            // Get the icon from the type descriptor
            var iconClass = formatters.contentIconClass(this.item.typeIdentifier);
            domClass.add(this.iconNode, iconClass);

            node.innerHTML = this.item.get("rolesString");
            this.own(this.item.watch("rolesString", lang.hitch(this, function (property, oldValue, newValue) {
                node.innerHTML = newValue;
            })));

            this.own(this.item.watch("readOnly", lang.hitch(this, function () {
                this.set("readOnly", this.item.readOnly);
            })));

            this.own(on(this.item, "changed", function () {
                this.item.updateTooltip();
                this.set("item", this.item);
                this.set("label", this.item.label);
                this.set("tooltip", this.item.tooltip);
            }.bind(this)));

            if (this.item.isBrokenLink) {
                domClass.add(this.iconNode, "epi-iconWarning");
            }
        },

        setSelected: function (selected, internalSelect) {
            this.inherited(arguments);

            this.item.set("selected", selected);

            domClass.toggle(this.iconNodeMenu, this._contextMenuClass, selected);
        },

        showPersonalizationSelector: function () {
            if (this.readOnly) {
                return;
            }

            this.item.set("ensurePersonalization", false);

            focusUtil.focus(this.labelNode);

            this.loadAndOpenDropDown();
        },

        focus: function () {
            // summary:
            //      We need this method because _HasDropDown calls focus when toggling the dropdown
            this.labelNode.focus();
        },

        _onKey: function (/*Event*/ e) {
            // summary:
            //      We need to override the default behaviour otherwise the dropdown will popup
            //      when we click the downarrow on the tree node
            if (this.dropDown && this._opened && e.keyCode === keys.ESCAPE) {
                this.closeDropDown();
            }
        },

        _onKeyUp: function () {

        },

        _setItemAttr: function (item) {
            this._set("item", item);
            this.sharedIndicatorNode.classList.toggle("dijitHidden", !applicationSettings.inlineBlocksInContentAreaEnabled || (item && item.inlineBlockData));
        }
    });
});
