define([], function () {
    // summary:
    //      Common properties that are added to every tracked events.

    var commonProperties = {};

    function getResolutions() {
        var windowInnerWidth = window.innerWidth
            || document.documentElement.clientWidth
            || document.body.clientWidth;
        var windowInnerHeight = window.innerHeight
            || document.documentElement.clientHeight
            || document.body.clientHeight;

        var windowDimensions = windowInnerWidth + "x" + windowInnerHeight;
        var screenDimensions = screen.width + "x" + screen.height;

        return {
            screenWidth: screen.width,
            screenHeight: screen.height,
            screenDimensions: screenDimensions,
            windowInnerWidth: windowInnerWidth,
            windowInnerHeight: windowInnerHeight,
            windowDimensions: windowDimensions
        };
    }

    return {
        initialize: function (telemetry) {
            commonProperties = {
                user_hasAdminAccess: telemetry.user_hasAdminAccess,
                user_creationDate: telemetry.user_creationDate,
                user_uiCulture: telemetry.user_uiCulture
            };
            Object.assign(commonProperties, telemetry.versions);
            Object.assign(commonProperties, getResolutions());
            return commonProperties;
        },

        set: function (properties) {
            // summary:
            //      Add or update common properties
            //
            // properties: Object
            //      the properties to be added/updated

            if (typeof properties !== "object") {
                return;
            }
            Object.assign(commonProperties, properties);
        }
    };
});
