define([
    "dojo/when",
    "epi/dependency",
    "epi/shell/applicationSettings",
    "epi-cms/telemetry/commonProperties",
    "epi-cms/telemetry/trackComponents",
    "epi-cms/telemetry/trackProjects",
    "epi-cms/telemetry/trackContentCreation",
    "epi-cms/telemetry/trackInlineMainButton",
    "epi-cms/telemetry/trackPanes",
    "epi-cms/telemetry/trackQuickEdit",
    "epi-cms/telemetry/trackEditMode",
    "epi-cms/telemetry/tracker"
], function (
    when,
    dependency,
    applicationSettings,
    commonProperties,
    trackComponents,
    trackProjects,
    trackContentCreation,
    trackInlineMainButton,
    trackPanes,
    trackQuickEdit,
    trackEditMode,
    tracker
) {
    return {
        initialize: function () {
            if (typeof aptrinsic !== "function") {
                return;
            }

            var telemetry = applicationSettings.telemetry;
            if (!telemetry) {
                return;
            }

            var registry = dependency.resolve("epi.storeregistry");
            var componentsStore = registry.get("epi.shell.component");
            //Initialize Gainsight
            var userFields = {
                id: telemetry.user,
                signUpDate: telemetry.user_creationDate
            };
            var accountFields = {
                id: telemetry.client
            };

            // current using gadgets component
            componentsStore.query({ viewName: "/episerver/cms/home" }).then(function (components) {
                userFields.cms_current_using_gadget = trackComponents.filterCurrentUsingGadgets(components);
            }).always(function () {
                // eslint-disable-next-line no-undef
                aptrinsic("identify",
                    userFields,
                    accountFields
                );
            });

            tracker.initialize(telemetry);
            tracker.track("telemetryInitializer", "Client", commonProperties.initialize(telemetry));

            trackComponents.initialize();
            trackProjects.initialize();
            trackContentCreation();
            trackInlineMainButton.initialize();
            trackPanes();
            trackQuickEdit.initialize();
            trackEditMode();
        }
    };
});
