define([
    "dojo/topic",
    "epi/shell/widget/ComponentController",
    "epi-cms/ApplicationSettings",
    "epi-cms/telemetry/tracker"
], function (
    topic,
    ComponentController,
    ApplicationSettings,
    tracker
) {

    //TODO TELEMETRY: this code can now be simplified, all prototype hacks can be removed

    function filterGadgets(components) {
        // filter data to exclude default components, we track direct children of ComponentPaneContainers
        // AND Navigation, Assets panes
        var pinnablePaneDefinition = "EPiServer.Shell.ViewComposition.Containers.PinnablePane";
        var componentPaneDefinition = "EPiServer.Shell.ViewComposition.Containers.ComponentPaneContainer";
        var componentGroupDefinition = "EPiServer.Shell.ViewComposition.Containers.ComponentGroup";

        var pinnablePaneIds = components.filter(function (component) {
            return component.definitionName === pinnablePaneDefinition;
        }).map(function (e) {
            return e.id;
        });

        var componentPaneIds = components.filter(function (component) {
            return component.definitionName === componentPaneDefinition && pinnablePaneIds.includes(component.parentId);
        }).map(function (e) {
            return e.id;
        });

        var filteredComponents = components.filter(function (component) {
            return componentPaneIds.includes(component.parentId) && component.definitionName !== componentGroupDefinition;
        }).map(function (component) {
            return component.definitionName;
        });

        // track Navigation and Assets panes usage (visibility) by ComponentGroup"s headingLocalizationKey and isHidden setting
        // "/episerver/cms/homeview/navigation", "/episerver/cms/homeview/assets"
        components.forEach(function (component) {
            var settings = component.settings || {};
            if (settings.headingLocalizationKey === "/episerver/cms/homeview/navigation" && !settings.isHidden) {
                filteredComponents.push("EPiServer.Cms.Shell.UI.Components.MainNavigationComponent");
            }
            if (settings.headingLocalizationKey === "/episerver/cms/homeview/assets" && !settings.isHidden) {
                filteredComponents.push("EPiServer.Cms.Shell.UI.Components.AssetsComponent");
            }
        });

        //distinct result
        return filteredComponents.filter(function (value, index, self) {
            return self.indexOf(value) === index;
        }).join(",");
    }

    function patchComponentController() {
        var originalAddComponent = ComponentController.prototype.addComponent;
        ComponentController.prototype.addComponent = function (container, component, createComponent, callback) {
            var result = originalAddComponent.apply(this, arguments);
            tracker.track("trackComponent", "Gadget Added", { name: component.definitionName });
            return result;
        };
        ComponentController.prototype.addComponent.nom = "addComponent";

        var originalRemoveComponent = ComponentController.prototype.removeComponent;
        ComponentController.prototype.removeComponent = function (identity) {
            var result = originalRemoveComponent.apply(this, arguments);
            var component = this.getComponentDefinitionById(identity);
            tracker.track("trackComponent", "Gadget Removed", { name: component.definitionName });
            return result;
        };
        ComponentController.prototype.removeComponent.nom = "removeComponent";
    }

    function onToggleComponent(data) {
        //track removing Main Navigation and Assets, adding (hiding) event is tracked in ComponentController.addComponent action
        if (data && data.isHidden) {
            if (data.key === ApplicationSettings.mainNavigationComponentKey ||
                data.key === "/episerver/cms/homeview/navigation") {
                tracker.track("trackComponent", "Gadget Removed", { name: "EPiServer.Cms.Shell.UI.Components.MainNavigationComponent" });
            }

            if (data.key === ApplicationSettings.assetsComponentKey ||
                data.key === "/episerver/cms/homeview/assets") {
                tracker.track("trackComponent", "Gadget Removed", { name: "EPiServer.Cms.Shell.UI.Components.AssetsComponent" });
            }
        }
    }

    return {
        initialize: function () {
            patchComponentController();
            topic.subscribe("/epi/cms/action/togglecomponent", onToggleComponent);
        },
        filterCurrentUsingGadgets: filterGadgets
    };
});
