define([
    "dojo/topic",
    "dojo/Deferred",
    "epi/dependency",
    "epi-cms/project/command/AddProject",
    "epi-cms/project/viewmodels/ProjectModeToolbarViewModel",
    "epi-cms/telemetry/tracker"
], function (
    topic,
    Deferred,
    dependency,
    AddProject,
    ProjectModeToolbarViewModel,
    tracker
) {
    function patchAddProjectCommand() {
        var originalOnDialogExecute = AddProject.prototype.onDialogExecute;
        AddProject.prototype.onDialogExecute = function () {
            originalOnDialogExecute.apply(this, arguments);
            tracker.track("trackProjects", "Project Create");
        };
        AddProject.prototype.onDialogExecute.nom = "onDialogExecute";
    }

    function patchRemoveProjectCommand() {
        var originalRemoveProject = ProjectModeToolbarViewModel.prototype.removeProject;
        ProjectModeToolbarViewModel.prototype.removeProject = function () {
            originalRemoveProject.apply(this, arguments);
            tracker.track("trackProjects", "Project Delete");
        };
        ProjectModeToolbarViewModel.prototype.removeProject.nom = "removeProject";
    }

    return {
        // Returns a promise object that is resolved with the bool "isProjectSelected"
        getProjectState: function () {
            var serviceId = "epi.cms.ProjectService";
            if (dependency.exists(serviceId)) {
                var projectService = dependency.resolve(serviceId);
                return projectService.getCurrentProjectId()
                    .then(function (projectId) {
                        return projectId != null;
                    });
            } else {
                return new Deferred(false);
            }
        },

        initialize: function () {
            var currentContextUri;

            topic.subscribe("/epi/shell/context/request", function (args) {
                var newContextUri = args.uri;
                if (newContextUri && newContextUri.indexOf("epi.cms.project:///") !== -1 && newContextUri !== currentContextUri) {
                    tracker.track("trackProjects", "Project Overview");
                }

                currentContextUri = newContextUri;
            });

            patchAddProjectCommand();

            patchRemoveProjectCommand();
        }
    };
});
