define([
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",

    // dijit
    "dijit/registry",
    "dijit/_CssStateMixin",
    "dijit/_TemplatedMixin",
    "dijit/_Widget",
    "dijit/focus",

    // epi.shell
    "epi/shell/command/_Command",
    "epi/shell/command/builder/ButtonBuilder",
    "epi/shell/command/builder/DropDownButtonBuilder",
    "epi/shell/command/builder/MenuAssembler",
    "epi/shell/TypeDescriptorManager",

    // epi.cms
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/widget/command/DefaultContentSelectorPlugAndPlayCommand",
    "epi-cms/plugin-area/content-selector-plugandplay",

    // resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselectorplugandplay",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselector",
    "dojo/text!./templates/ContentSelectorPlugAndPlay.html"
], function (
    // dojo
    declare,
    lang,
    aspect,

    // dijit
    registry,
    _CssStateMixin,
    _TemplatedMixin,
    _Widget,
    focusManager,

    // epi.shell
    _Command,
    ButtonBuilder,
    DropDownButtonBuilder,
    MenuAssembler,
    TypeDescriptorManager,

    // epi.cms
    _HasChildDialogMixin,
    DefaultContentSelectorPlugAndPlayCommand,
    plugInArea,

    // resources
    localization,
    contentselectorLocalization,
    template
) {
    return declare([_Widget, _TemplatedMixin, _CssStateMixin, _HasChildDialogMixin], {
        // summary:
        //    Represents the widget to select ContentReference with extra actions everywhere.
        // tags:
        //      internal xproduct

        baseClass: "epi-content-selector-plugnplay",
        templateString: template,
        resources: localization,

        iconClass: "iconUpload",
        text: null,
        commands: null,
        dialogSettings: null,
        contentSelectorDialogSettings: null,
        readonly: false,
        selectCommandText: null,
        buttonLabel: null,

        dialog: null,
        contentSelectorDialog: null,
        actionsNodeOptions: null,
        _selectContentCommand: null,
        _menuWithSeparator: null,

        onContentSelectorChange: function (/*object*/value) {
            // summary:
            //    Handle content selector inside dialog is changed value.
            // tags:
            //    public callback
        },

        onSearchResultSelected: function (/*object*/value) {
            // summary:
            //    Handle a search result is selected.
            // tags:
            //    public callback
        },

        onDialogExecute: function (/*object*/value) {
            // summary:
            //    Handle dialog execute
            // tags:
            //    public callback
        },

        onDialogShow: function () {
            // summary:
            //    Handle dialog show
            // tags:
            //    public callback
        },

        onDialogHide: function () {
            // summary:
            //    Handle dialog hide
            // tags:
            //    public callback
        },

        _getActionButton: function () {
            var node = this.actionsNode.children[0];
            return node && registry.byNode(node);
        },

        _setTitleAttr: function (value) {
            this._set("title", value);

            // update dialog title
            this.dialogSettings.title = value;
            this.dialog && this.dialog.set("title", value);
        },

        _setSelectCommandTextAttr: function (value) {
            this._set("selectCommandText", value);

            // update button label in case we have only one command.
            var button = this._getActionButton();
            button && this._set("buttonLabel", value);
            button && button.set("label", value);
        },

        _setReadOnlyAttr: function (value) {
            this._set("readOnly", value);
            [].concat(this.commands).forEach(function (cmd) {
                cmd && cmd.set("canExecute", !value);
            });
        },

        _setDisabledAttr: function (value) {
            this._set("disabled", value);
            [].concat(this.commands).forEach(function (cmd) {
                cmd && cmd.set("canExecute", !value);
            });
        },

        _setupCommands: function () {
            this.commands = this.commands || [];
            this._selectContentCommand = this._selectContentCommand || new DefaultContentSelectorPlugAndPlayCommand({
                contentSelectorDialogSettings: this.contentSelectorDialogSettings,
                dialogSettings: this.dialogSettings,
                label: this.selectCommandText || this.resources.selectcontentcommand.label,
                canExecute: !this.readonly
            });
            ["onContentSelectorChange", "onSearchResultSelected", "onDialogExecute", "onDialogHide", "onDialogShow"].forEach(function (callbackName) {
                this.own(aspect.after(this._selectContentCommand, callbackName, function () {
                    this[callbackName].apply(this, arguments);
                }.bind(this), true));
            }.bind(this));

            this._menuWithSeparator = this._menuWithSeparator || new _Command({ isAvailable: false, canExecute: false, category: "menuWithSeparator" });
            if (this.commands.length > 0) {
                this.commands.push(this._menuWithSeparator);
            }
            this.commands.push(this._selectContentCommand);
            [].concat(plugInArea.get()).forEach(function (cmd) {
                var otherCmd = lang.mixin({}, cmd);
                otherCmd.set("model", lang.mixin({
                    allowedTypes: this.contentSelectorDialogSettings.allowedTypes,
                    restrictedTypes: this.contentSelectorDialogSettings.restrictedTypes,
                    modelType: this.contentSelectorDialogSettings.modelType
                }, otherCmd.model || otherCmd.get("model") || {}));
                this.commands.push(otherCmd);
            }.bind(this));

            this.own(this.commands);
        },

        _onPopupOpen: function (openArgs) {
            this.inherited(arguments);
            if (this.actionsNode.contains(openArgs.around)) {
                this._openedChildDialog = openArgs.popup;
                this.isShowingChildDialog = true;
            }
        },

        _correctLabelByTypes: function () {
            // summary:
            //      Support
            //      1. Single base type: folder, block, page, image, video, media
            //      2. Multiple of 'media' like: image + video will be treated as 'media'
            //      3. Otherwise is Default = 'Select Content'
            // tags:
            //      private

            var types = TypeDescriptorManager.removeIntersectingTypes(
                this.contentSelectorDialogSettings.allowedTypes,
                this.contentSelectorDialogSettings.restrictedTypes);

            if (!types || types.length <= 0) {
                return;
            }

            var localization = contentselectorLocalization;
            var mediaTypeBase = { typeIdentifier: "episerver.core.icontentmedia", label: localization.button.selectmedia };
            var contentTypeBases = [
                { typeIdentifier: "episerver.core.contentfolder", label: localization.button.selectfolder },
                { typeIdentifier: "episerver.core.blockdata", label: localization.button.selectblock },
                { typeIdentifier: "episerver.core.pagedata", label: localization.button.selectpage },
                { typeIdentifier: "episerver.core.icontentimage", label: localization.button.selectimage },
                { typeIdentifier: "episerver.core.icontentvideo", label: localization.button.selectvideo },
                mediaTypeBase
            ];

            var found = types.map(function (t) {
                return contentTypeBases.find(function (b) {
                    return t === b.typeIdentifier || TypeDescriptorManager.isBaseTypeIdentifier(t, b.typeIdentifier);
                });
            }).filter(function (t) {
                return t;
            });

            if (!found || found.length !== 1) {
                // all is media?
                if (types.some(function (x) {
                    return x !== mediaTypeBase.typeIdentifier && !TypeDescriptorManager.isBaseTypeIdentifier(x, mediaTypeBase.typeIdentifier);
                })) {
                    return;
                }

                found = [mediaTypeBase];
            }

            var first = found[0];
            this.title = this.title || first.label;
            this.selectCommandText = this.selectCommandText || first.label;
            this.buttonLabel = this.buttonLabel || first.label;
        },

        getCommands: function () {
            // summary:
            //      Return commands for menu assembler.
            return this.commands;
        },

        postMixInProperties: function () {
            // summary:
            //      Initialize properties
            // tags:
            //      protected

            this.inherited(arguments);

            this.contentSelectorDialogSettings = Object.assign({
                roots: this.roots,
                allowedTypes: ["episerver.core.icontent", "episerver.core.icontentdata"],
                canSelectOwnerContent: false,
                getCurrentValue: function () {
                    return undefined;
                },
                isInQuickEditMode: this.isInQuickEditMode,
                quickEditBlockId: this.quickEditBlockId
            }, this.contentSelectorDialogSettings || {});
            this._correctLabelByTypes();

            this.dialogSettings = Object.assign({
                title: this.title || this.resources.dialogtitle,
                dialogClass: "epi-dialog-portrait epi-content-selector-plugnplay-dialog",
                destroyOnHide: true
            }, this.dialogSettings || {});

            this.text = this.text || this.resources.text;

            this._setupCommands();

            this.actionsNodeOptions = Object.assign({
                itemSettings: {
                    showLabel: true
                },
                settings: {
                    label: this.buttonLabel || this.resources.buttonlabel,
                    showLabel: true
                },
                actionsBuilder: this.commands.filter(function (cmd) {
                    return cmd.isAvailable;
                }).length > 1 ? DropDownButtonBuilder : ButtonBuilder,
                commandSource: this
            }, this.actionsNodeOptions || {});
            // avoid null or empty label
            this.actionsNodeOptions.settings.label = this.actionsNodeOptions.settings.label || this.resources.buttonlabel;
        },

        postCreate: function () {
            // summary:
            //      Construct commands list and build icon, text, actions button.
            // tags:
            //      protected

            this.inherited(arguments);

            this.buildIconNode(this.iconNode, this.iconClass);
            this.buildTextNode(this.textNode, this.text);
            this.buildActionsNode(this.actionsNode, this.actionsNodeOptions);

            var actionButton = this._getActionButton();
            actionButton && this.own(
                actionButton.on("click", function (evt) {
                    evt.preventDefault();
                    evt.stopPropagation();
                    return false;
                }.bind(this))
            );

            // default endpoint for command raise saved value to editor.
            [].concat(this.commands).forEach(function (cmd) {
                this.own(aspect.after(cmd, "onExecuted", function (contentLink) {
                    this.onDialogExecute(contentLink);
                }.bind(this), true));
            }.bind(this));

            // correct status
            var canExecute = !this.get("readOnly") && !this.get("disabled");
            [].concat(this.commands).forEach(function (cmd) {
                cmd && cmd.set("canExecute", canExecute);
            }.bind(this));
        },

        focus: function () {
            // summary:
            //       Put focus on this widget
            // tags:
            //      public

            var focusNode = this.focusNode || this.inputContainer;
            try {
                focusManager.focus(focusNode);
            } catch (e) {
                /*squelch errors from hidden nodes*/
            }
        },

        selectContent: function () {
            // summary:
            //      Helper method to manually show content selector dialog.
            // tags:
            //      protected

            if (!this.commands || this.commands.length > 1) {
                return;
            }

            var actionButton = this._getActionButton();
            actionButton && actionButton.onClick(new Event("click"));
        },

        buildIconNode: function (iconTargetNode, iconClass) {
            iconTargetNode.classList.add(iconClass);
        },

        buildTextNode: function (textTargetNode, text) {
            textTargetNode.innerText = text;
        },

        buildActionsNode: function (actionsTargetNode, options) {

            var builderSettings = {
                itemSettings: options.itemSettings,
                settings: options.settings
            };
            var actionsBuilder = new options.actionsBuilder(builderSettings),
                menuAssemblerConfiguration = [
                    { builder: actionsBuilder, target: actionsTargetNode },
                    { category: "menuWithSeparator", builder: actionsBuilder, target: actionsTargetNode }
                ],
                menuAssembler = new MenuAssembler({
                    configuration: menuAssemblerConfiguration,
                    commandSource: options.commandSource
                });

            menuAssembler.build();
        }
    });
});
